<?php

namespace Drupal\reaction\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\Unicode;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Path\PathValidatorInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Url;
use Drupal\Core\Session\AccountInterface;
use Drupal\link\LinkItemInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\field\Entity\FieldConfig;

/**
 * Plugin implementation of the 'reaction' formatter.
 *
 * @FieldFormatter(
 *   id = "reaction",
 *   label = @Translation("Reaction"),
 *   field_types = {
 *     "reaction"
 *   }
 * )
 */
class ReactionFormatter extends FormatterBase implements ContainerFactoryPluginInterface {

  /**
   * @var AccountInterface $account
   */
  protected $account;

  /**
   * The path validator service.
   *
   * @var \Drupal\Core\Path\PathValidatorInterface
   */
  protected $pathValidator;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
        $plugin_id, $plugin_definition, $configuration['field_definition'], $configuration['settings'], $configuration['label'], $configuration['view_mode'], $configuration['third_party_settings'], $container->get('path.validator'), $container->get('current_user'), $container->get('database')
    );
  }

  /**
   * Constructs a new LinkFormatter.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Third party settings.
   * @param \Drupal\Core\Path\PathValidatorInterface $path_validator
   *   The path validator service.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, PathValidatorInterface $path_validator, AccountInterface $account) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->pathValidator = $path_validator;
    $this->account = $account;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return array(
      'reaction_style' => 'google',
      'reaction_position' => 'horizontal',
      'reaction_css_class_name' => 'reaction',
      'reaction_layout_style' => 'simple_layout',
        ) + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    global $base_url;
    $module_path = drupal_get_path('module', 'reaction');
    $elements = parent::settingsForm($form, $form_state);
    $path = drupal_get_path('module', 'reaction');
    if ($handle = opendir($path . '/vendor/krknth/reaction/')) {
      while (false !== ($entry = readdir($handle))) {
        if ($entry != "." && $entry != ".." && $entry != "." && $entry != ".git" && $entry != "Readme.txt" && $entry != "composer.json") {
          $reaction_style_option[$entry] = ucfirst($entry);
        }
      }
      closedir($handle);
    }
    $imageOptions = array(
      "simple_layout" => '<img src = ' . $base_url . '/' . $module_path . '/styles/icons/simple_layout.png>',
      "fancy_layout" => '<img src = ' . $base_url . '/' . $module_path . '/styles/icons/fancy_layout.png>',
    );
    $elements['reaction_style'] = [
      '#type' => 'select',
      '#title' => t('Select the type of style'),
      '#options' => $reaction_style_option,
      '#default_value' => $this->getSetting('reaction_style'),
    ];
    $elements['reaction_position'] = array(
      '#type' => 'select',
      '#title' => t('Select the position'),
      '#options' => array('vertical' => 'Vertical', 'horizontal' => 'Horizontal'),
      '#default_value' => $this->getSetting('reaction_position'),
    );
    $elements['reaction_css_class_name'] = array(
      '#type' => 'textfield',
      '#title' => t('Css Class name'),
      '#default_value' => $this->getSetting('reaction_css_class_name'),
    );
    $elements['reaction_layout_style'] = array(
      '#type' => 'radios',
      '#options' => $imageOptions,
      '#title' => t('Choose the Layout'),
      '#default_value' => $this->getSetting('reaction_layout_style'),
    );
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = array();

    $settings = $this->getSettings();
    if (!empty($settings['reaction_style'])) {
      $summary[] = t('Reaction is styled to @limit', array('@limit' => $settings['reaction_style']));
    }
    else {
      $summary[] = t('Reaction is not styled');
    }
    if (!empty($settings['reaction_position'])) {
      $summary[] = t('Reaction is position at @limit', array('@limit' => $settings['reaction_position']));
    }
    else {
      $summary[] = t('Reaction is not positioned');
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    global $base_url;
    $module_path = drupal_get_path('module', 'reaction');
    $reacted = FALSE;
    $user_id = $this->account->id();
    $entity = $items->getEntity();
    $field_definition = $items->getFieldDefinition();
    $entity_type = $entity->getEntityTypeId();
    $bundle = $entity->bundle();
    $entity_id = $entity->id();
    $field_name = $field_definition->getName();
    $reaction_emoji_display = $this->getFieldSetting('reaction_style');
    foreach ($reaction_emoji_display as $emoji_name) {
      if (!is_numeric($emoji_name)) {
       $emoji_array[$emoji_name] = $emoji_name;
      }
    }
    //$emoji_array = explode(",", $reaction_emoji_display);
    $layout_style = $this->getSetting('reaction_layout_style');
    $ip = \Drupal::request()->getClientIp();
    //$field_info = FieldConfig::loadByName('node', 'page', 'field_reaction');
    // Get the private key.
    //$private_key = \Drupal::service('private_key')->get();
    // Get a drupal CSRF token. (The actual service is called 'csrf_token').
    $token_generator = \Drupal::csrfToken();
    $token = $token_generator->get();
    //reaction/{entity_type}/{bundle}/{field}/{id}/{expression}/{token}
    $style_setting = $this->getSetting('reaction_style');
    $query = db_select('reaction_user_track', 'rut');
    $query->join('reaction', 'r', 'rut.rut_id = r.rut_id');
    $query->fields('r', array('eid'));
    $query->fields('rut', array('uid'));
    $query->condition('rut.uid', $user_id);
    $query->condition('rut.ip_address', $ip);
    $query->condition('r.eid', $entity_id);
    $result = $query->execute()->fetchAll();
    if (count($result) >= 1) {
      $reacted = TRUE;
    }
    //To get the total count which we are using in calculate percentage
    foreach ($emoji_array as $emoji) {
      $count = _get_count_reaction($entity_id, $emoji);
      $total_count +=$count;
    }
    //formula used for calculate percentage
    // 100/Total count number of reacted result and multiplying the result
    // with count of each emoji to get the percentage
    foreach ($emoji_array as $emoji) {
      $user_reacted_emoji_class = _get_reacted_emoji_class($entity_id, $emoji, $user_id);
      $count = _get_count_reaction($entity_id, $emoji);
      $percentage_calculate = 100 / $total_count;
      $url_data = ['entity_type' => $entity_type, 'bundle' => $bundle, 'field_name' => $field_name, 'entity_id' => $entity_id, 'expression' => $emoji, 'token' => $token, 'ip' => $ip, 'reacted_count' => $count, 'reaction_emoji_display' => $emoji_array, 'style_setting' => $style_setting, 'layout_style' => $layout_style];
      //encode with strtr since the url is appending with '=' to remove that
      //refer http://php.net/manual/en/function.base64-encode.php#103849
      $json_data = json_encode($url_data);
      $enocode_data = rtrim(strtr(base64_encode($json_data), '+/', '-_'), '=');
      $url = Url::fromRoute(
              'ajax_link.response', ['url_encode_data' => $enocode_data]
      );
      $image[] = array(
        'image_path' => $base_url . '/' . $module_path . '/vendor/krknth/reaction/' . $style_setting . '/' . $emoji . '.png',
        'image_url' => $url,
        'reacted_count' => $count,
        'user_reacted_emoji' => $user_reacted_emoji_class,
        'percentage' => is_nan(round($count * $percentage_calculate)) ? 0 : round($count * $percentage_calculate),
        'emoji_name' => $emoji, 
      );
    }
    $elements[] = array(
      '#theme' => 'reaction_formatter',
      '#style' => $style_setting,
      '#image' => $image,
      '#layout_style' => $layout_style,
      '#reacted' => $reacted,
    );
    \Drupal::service('page_cache_kill_switch')->trigger();
    //To disable the cache for authenticate user
    $elements['#cache']['max-age'] = 0;
    return $elements;
  }

  /*
    if ($reacted) {
    foreach ($emoji_array as $emoji) {
    $query = db_select('reaction_user_track', 'rut');
    $query->join('reaction', 'r', 'rut.rut_id = r.rut_id');
    $query->fields('r', array('value'));
    $query->condition('r.eid', $entity_id);
    $query->condition('r.value', $emoji);
    $result_array = $query->execute()->fetchAll();
    $count = count($result_array);
    $url = Url::fromRoute(
    'ajax_link.response', ['entity_type' => $entity_type, 'bundle' => $bundle, 'field_name' => $field_name, 'entity_id' => $entity_id, 'expression' => $emoji, 'token' => $token, 'ip' => $ip]
    );
    $image[] = array(
    'image_path' => '../modules/reaction/expressions/' . $style_setting . '/' . $emoji . '.png',
    'image_url' => $url
    );
    }
    $elements[] = array(
    '#theme' => 'reaction_formatter',
    '#style' => $style_setting,
    '#image' => $image,
    '#layout_style' => $layout_style,
    );
    }
    else {
    foreach ($emoji_array as $emoji) {
    $query = db_select('reaction_user_track', 'rut');
    $query->join('reaction', 'r', 'rut.rut_id = r.rut_id');
    $query->fields('r', array('value'));
    $query->condition('r.eid', $entity_id);
    $query->condition('r.value', $emoji);
    $result_array = $query->execute()->fetchAll();
    $count = count($result_array);
    $image[] = array(
    'image_path' => '../modules/reaction/expressions/' . $style_setting . '/' . $emoji . '.png',
    'reacted' => $count,
    );
    }
    $elements[] = array(
    '#theme' => 'reacted_formatter',
    '#style' => $style_setting,
    '#image' => $image,
    '#layout_style' => $layout_style,
    );
    } */
  /* $image = array(
    array(
    'image_path' => '../sites/default/files/expressions/' . $style_setting . '/smiling-face-with-sunglasses.png',
    'image_url' => $url
    ),
    array(
    'image_path' => '../sites/default/files/expressions/' . $style_setting . '/white-smiling-face.png',
    'image_url' => $url
    ),
    ); */
  //TODO: Need to inject the service
  //To disable the cache for anonymous user
  // }

  /* $element = array();
    $entity = $items->getEntity();
    $settings = $this->getSettings();

    foreach ($items as $delta => $item) {
    // By default use the full URL as the link text.
    $url = $this->buildUrl($item);
    $link_title = $url->toString();

    // If the title field value is available, use it for the link text.
    if (empty($settings['url_only']) && !empty($item->title)) {
    // Unsanitized token replacement here because the entire link title
    // gets auto-escaped during link generation in
    // \Drupal\Core\Utility\LinkGenerator::generate().
    $link_title = \Drupal::token()->replace($item->title, [$entity->getEntityTypeId() => $entity], ['clear' => TRUE]);
    }

    // Trim the link text to the desired length.
    if (!empty($settings['trim_length'])) {
    $link_title = Unicode::truncate($link_title, $settings['trim_length'], FALSE, TRUE);
    }

    if (!empty($settings['url_only']) && !empty($settings['url_plain'])) {
    $element[$delta] = array(
    '#plain_text' => $link_title,
    );

    if (!empty($item->_attributes)) {
    // Piggyback on the metadata attributes, which will be placed in the
    // field template wrapper, and set the URL value in a content
    // attribute.
    // @todo Does RDF need a URL rather than an internal URI here?
    // @see \Drupal\Tests\rdf\Kernel\Field\LinkFieldRdfaTest.
    $content = str_replace('internal:/', '', $item->uri);
    $item->_attributes += array('content' => $content);
    }
    }
    else {
    $element[$delta] = array(
    '#type' => 'link',
    '#title' => $link_title,
    '#options' => $url->getOptions(),
    );
    $element[$delta]['#url'] = $url;

    if (!empty($item->_attributes)) {
    $element[$delta]['#options'] += array('attributes' => array());
    $element[$delta]['#options']['attributes'] += $item->_attributes;
    // Unset field item attributes since they have been included in the
    // formatter output and should not be rendered in the field template.
    unset($item->_attributes);
    }
    }
    }

    return $element;
    }
   */
}
